@@ -1,19 +1,24 @@
-import os
-from ctypes import (
-    ArgumentError,
-    byref,
-    c_char,
-    c_long,
-    c_uint,
-    c_ulong,
-    pointer,
-    windll,
-)
-from ctypes.wintypes import DWORD, HANDLE
-from typing import Dict, List, Optional, TextIO, Tuple
+from __future__ import annotations
 
+import sys
+
+assert sys.platform == "win32"
+
+import os
+from ctypes import ArgumentError, byref, c_char, c_long, c_uint, c_ulong, pointer
+
+from ..utils import SPHINX_AUTODOC_RUNNING
+
+# Do not import win32-specific stuff when generating documentation.
+# Otherwise RTD would be unable to generate docs for this module.
+if not SPHINX_AUTODOC_RUNNING:
+    from ctypes import windll
+
+from ctypes.wintypes import DWORD, HANDLE
+from typing import Callable, Dict, List, Optional, TextIO, Tuple, Type, TypeVar, Union
+
+from prompt_toolkit.cursor_shapes import CursorShape
 from prompt_toolkit.data_structures import Size
-from prompt_toolkit.renderer import Output
 from prompt_toolkit.styles import ANSI_COLOR_NAMES, Attrs
 from prompt_toolkit.utils import get_cwidth
 from prompt_toolkit.win32_types import (
@@ -24,6 +29,7 @@ from prompt_toolkit.win32_types import (
     STD_OUTPUT_HANDLE,
 )
 
+from .base import Output
 from .color_depth import ColorDepth
 
 __all__ = [
@@ -31,7 +37,7 @@ __all__ = [
 ]
 
 
-def _coord_byval(coord):
+def _coord_byval(coord: COORD) -> c_long:
     """
     Turns a COORD object into a c_long.
     This will cause it to be passed by value instead of by reference. (That is what I think at least.)
@@ -79,6 +85,9 @@ class NoConsoleScreenBufferError(Exception):
         super().__init__(message)
 
 
+_T = TypeVar("_T")
+
+
 class Win32Output(Output):
     """
     I/O abstraction for rendering to Windows consoles.
@@ -89,12 +98,12 @@ class Win32Output(Output):
         self,
         stdout: TextIO,
         use_complete_width: bool = False,
-        default_color_depth: Optional[ColorDepth] = None,
+        default_color_depth: ColorDepth | None = None,
     ) -> None:
         self.use_complete_width = use_complete_width
         self.default_color_depth = default_color_depth
 
-        self._buffer: List[str] = []
+        self._buffer: list[str] = []
         self.stdout: TextIO = stdout
         self.hconsole = HANDLE(windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE))
 
@@ -111,11 +120,11 @@ class Win32Output(Output):
             self.LOG = open(_DEBUG_RENDER_OUTPUT_FILENAME, "ab")
 
     def fileno(self) -> int:
-        " Return file descriptor. "
+        "Return file descriptor."
         return self.stdout.fileno()
 
     def encoding(self) -> str:
-        " Return encoding used for stdout. "
+        "Return encoding used for stdout."
         return self.stdout.encoding
 
     def write(self, data: str) -> None:
@@ -125,7 +134,7 @@ class Win32Output(Output):
         self._buffer.append(data)
 
     def write_raw(self, data: str) -> None:
-        " For win32, there is no difference between write and write_raw. "
+        "For win32, there is no difference between write and write_raw."
         self.write(data)
 
     def get_size(self) -> Size:
@@ -148,7 +157,7 @@ class Win32Output(Output):
         # Create `Size` object.
         return Size(rows=height, columns=width)
 
-    def _winapi(self, func, *a, **kw):
+    def _winapi(self, func: Callable[..., _T], *a: object, **kw: object) -> _T:
         """
         Flush and call win API function.
         """
@@ -170,11 +179,11 @@ class Win32Output(Output):
             return func(*a, **kw)
         except ArgumentError as e:
             if _DEBUG_RENDER_OUTPUT:
-                self.LOG.write(
-                    ("    Error in %r %r %s\n" % (func.__name__, e, e)).encode("utf-8")
-                )
+                self.LOG.write((f"    Error in {func.__name__!r} {e!r} {e}\n").encode())
 
-    def get_win32_screen_buffer_info(self):
+            raise
+
+    def get_win32_screen_buffer_info(self) -> CONSOLE_SCREEN_BUFFER_INFO:
         """
         Return Screen buffer info.
         """
@@ -244,7 +253,7 @@ class Win32Output(Output):
 
         self._erase(start, length)
 
-    def _erase(self, start, length):
+    def _erase(self, start: COORD, length: int) -> None:
         chars_written = c_ulong()
 
         self._winapi(
@@ -268,14 +277,24 @@ class Win32Output(Output):
         )
 
     def reset_attributes(self) -> None:
-        " Reset the console foreground/background color. "
+        "Reset the console foreground/background color."
         self._winapi(
             windll.kernel32.SetConsoleTextAttribute, self.hconsole, self.default_attrs
         )
         self._hidden = False
 
     def set_attributes(self, attrs: Attrs, color_depth: ColorDepth) -> None:
-        fgcolor, bgcolor, bold, underline, italic, blink, reverse, hidden = attrs
+        (
+            fgcolor,
+            bgcolor,
+            bold,
+            underline,
+            strike,
+            italic,
+            blink,
+            reverse,
+            hidden,
+        ) = attrs
         self._hidden = bool(hidden)
 
         # Start from the default attributes.
@@ -311,14 +330,14 @@ class Win32Output(Output):
         pass
 
     def cursor_goto(self, row: int = 0, column: int = 0) -> None:
-        pos = COORD(x=column, y=row)
+        pos = COORD(X=column, Y=row)
         self._winapi(
             windll.kernel32.SetConsoleCursorPosition, self.hconsole, _coord_byval(pos)
         )
 
     def cursor_up(self, amount: int) -> None:
         sr = self.get_win32_screen_buffer_info().dwCursorPosition
-        pos = COORD(sr.X, sr.Y - amount)
+        pos = COORD(X=sr.X, Y=sr.Y - amount)
         self._winapi(
             windll.kernel32.SetConsoleCursorPosition, self.hconsole, _coord_byval(pos)
         )
@@ -330,7 +349,7 @@ class Win32Output(Output):
         sr = self.get_win32_screen_buffer_info().dwCursorPosition
         #        assert sr.X + amount >= 0, 'Negative cursor position: x=%r amount=%r' % (sr.X, amount)
 
-        pos = COORD(max(0, sr.X + amount), sr.Y)
+        pos = COORD(X=max(0, sr.X + amount), Y=sr.Y)
         self._winapi(
             windll.kernel32.SetConsoleCursorPosition, self.hconsole, _coord_byval(pos)
         )
@@ -475,6 +494,12 @@ class Win32Output(Output):
     def show_cursor(self) -> None:
         pass
 
+    def set_cursor_shape(self, cursor_shape: CursorShape) -> None:
+        pass
+
+    def reset_cursor_shape(self) -> None:
+        pass
+
     @classmethod
     def win32_refresh_window(cls) -> None:
         """
@@ -532,8 +557,10 @@ class BACKGROUND_COLOR:
     INTENSITY = 0x0080  # Background color is intensified.
 
 
-def _create_ansi_color_dict(color_cls) -> Dict[str, int]:
-    " Create a table that maps the 16 named ansi colors to their Windows code. "
+def _create_ansi_color_dict(
+    color_cls: type[FOREGROUND_COLOR] | type[BACKGROUND_COLOR],
+) -> dict[str, int]:
+    "Create a table that maps the 16 named ansi colors to their Windows code."
     return {
         "ansidefault": color_cls.BLACK,
         "ansiblack": color_cls.BLACK,
@@ -573,10 +600,10 @@ class ColorLookupTable:
         self._win32_colors = self._build_color_table()
 
         # Cache (map color string to foreground and background code).
-        self.best_match: Dict[str, Tuple[int, int]] = {}
+        self.best_match: dict[str, tuple[int, int]] = {}
 
     @staticmethod
-    def _build_color_table() -> List[Tuple[int, int, int, int, int]]:
+    def _build_color_table() -> list[tuple[int, int, int, int, int]]:
         """
         Build an RGB-to-256 color conversion table
         """
@@ -602,7 +629,7 @@ class ColorLookupTable:
             (0xFF, 0xFF, 0xFF, FG.GRAY | FG.INTENSITY, BG.GRAY | BG.INTENSITY),
         ]
 
-    def _closest_color(self, r: int, g: int, b: int) -> Tuple[int, int]:
+    def _closest_color(self, r: int, g: int, b: int) -> tuple[int, int]:
         distance = 257 * 257 * 3  # "infinity" (>distance from #000000 to #ffffff)
         fg_match = 0
         bg_match = 0
@@ -620,7 +647,7 @@ class ColorLookupTable:
                 distance = d
         return fg_match, bg_match
 
-    def _color_indexes(self, color: str) -> Tuple[int, int]:
+    def _color_indexes(self, color: str) -> tuple[int, int]:
         indexes = self.best_match.get(color, None)
         if indexes is None:
             try:
