@@ -24,7 +24,9 @@ Example::
         s = PromptSession()
         result = s.prompt('Say something: ')
 """
-from asyncio import get_event_loop
+from __future__ import annotations
+
+from asyncio import get_running_loop
 from contextlib import contextmanager
 from enum import Enum
 from functools import partial
@@ -47,6 +49,11 @@ from prompt_toolkit.auto_suggest import AutoSuggest, DynamicAutoSuggest
 from prompt_toolkit.buffer import Buffer
 from prompt_toolkit.clipboard import Clipboard, DynamicClipboard, InMemoryClipboard
 from prompt_toolkit.completion import Completer, DynamicCompleter, ThreadedCompleter
+from prompt_toolkit.cursor_shapes import (
+    AnyCursorShapeConfig,
+    CursorShapeConfig,
+    DynamicCursorShapeConfig,
+)
 from prompt_toolkit.document import Document
 from prompt_toolkit.enums import DEFAULT_BUFFER, SEARCH_BUFFER, EditingMode
 from prompt_toolkit.filters import (
@@ -149,7 +156,7 @@ E = KeyPressEvent
 
 def _split_multiline_prompt(
     get_prompt_text: _StyleAndTextTuplesCallable,
-) -> Tuple[
+) -> tuple[
     Callable[[], bool], _StyleAndTextTuplesCallable, _StyleAndTextTuplesCallable
 ]:
     """
@@ -342,6 +349,7 @@ class PromptSession(Generic[_T]):
         "style_transformation",
         "swap_light_and_dark_colors",
         "color_depth",
+        "cursor",
         "include_default_pygments_style",
         "rprompt",
         "multiline",
@@ -380,38 +388,38 @@ class PromptSession(Generic[_T]):
         validate_while_typing: FilterOrBool = True,
         enable_history_search: FilterOrBool = False,
         search_ignore_case: FilterOrBool = False,
-        lexer: Optional[Lexer] = None,
+        lexer: Lexer | None = None,
         enable_system_prompt: FilterOrBool = False,
         enable_suspend: FilterOrBool = False,
         enable_open_in_editor: FilterOrBool = False,
-        validator: Optional[Validator] = None,
-        completer: Optional[Completer] = None,
+        validator: Validator | None = None,
+        completer: Completer | None = None,
         complete_in_thread: bool = False,
         reserve_space_for_menu: int = 8,
         complete_style: CompleteStyle = CompleteStyle.COLUMN,
-        auto_suggest: Optional[AutoSuggest] = None,
-        style: Optional[BaseStyle] = None,
-        style_transformation: Optional[StyleTransformation] = None,
+        auto_suggest: AutoSuggest | None = None,
+        style: BaseStyle | None = None,
+        style_transformation: StyleTransformation | None = None,
         swap_light_and_dark_colors: FilterOrBool = False,
-        color_depth: Optional[ColorDepth] = None,
+        color_depth: ColorDepth | None = None,
+        cursor: AnyCursorShapeConfig = None,
         include_default_pygments_style: FilterOrBool = True,
-        history: Optional[History] = None,
-        clipboard: Optional[Clipboard] = None,
-        prompt_continuation: Optional[PromptContinuationText] = None,
+        history: History | None = None,
+        clipboard: Clipboard | None = None,
+        prompt_continuation: PromptContinuationText | None = None,
         rprompt: AnyFormattedText = None,
         bottom_toolbar: AnyFormattedText = None,
         mouse_support: FilterOrBool = False,
-        input_processors: Optional[List[Processor]] = None,
-        placeholder: Optional[AnyFormattedText] = None,
-        key_bindings: Optional[KeyBindingsBase] = None,
+        input_processors: list[Processor] | None = None,
+        placeholder: AnyFormattedText | None = None,
+        key_bindings: KeyBindingsBase | None = None,
         erase_when_done: bool = False,
-        tempfile_suffix: Optional[Union[str, Callable[[], str]]] = ".txt",
-        tempfile: Optional[Union[str, Callable[[], str]]] = None,
+        tempfile_suffix: str | Callable[[], str] | None = ".txt",
+        tempfile: str | Callable[[], str] | None = None,
         refresh_interval: float = 0,
-        input: Optional[Input] = None,
-        output: Optional[Output] = None,
+        input: Input | None = None,
+        output: Output | None = None,
     ) -> None:
-
         history = history or InMemoryHistory()
         clipboard = clipboard or InMemoryClipboard()
 
@@ -436,6 +444,7 @@ class PromptSession(Generic[_T]):
         self.style_transformation = style_transformation
         self.swap_light_and_dark_colors = swap_light_and_dark_colors
         self.color_depth = color_depth
+        self.cursor = cursor
         self.include_default_pygments_style = include_default_pygments_style
         self.rprompt = rprompt
         self.multiline = multiline
@@ -751,6 +760,7 @@ class PromptSession(Generic[_T]):
             erase_when_done=erase_when_done,
             reverse_vi_search_direction=True,
             color_depth=lambda: self.color_depth,
+            cursor=DynamicCursorShapeConfig(lambda: self.cursor),
             refresh_interval=self.refresh_interval,
             input=self._input,
             output=self._output,
@@ -807,6 +817,7 @@ class PromptSession(Generic[_T]):
             display_completions_like_readline(event)
 
         @handle("c-c", filter=default_focused)
+        @handle("<sigint>")
         def _keyboard_interrupt(event: E) -> None:
             "Abort when Control-C has been pressed."
             event.app.exit(exception=KeyboardInterrupt, style="class:aborting")
@@ -845,50 +856,52 @@ class PromptSession(Generic[_T]):
         self,
         # When any of these arguments are passed, this value is overwritten
         # in this PromptSession.
-        message: Optional[AnyFormattedText] = None,
+        message: AnyFormattedText | None = None,
         # `message` should go first, because people call it as
         # positional argument.
         *,
-        editing_mode: Optional[EditingMode] = None,
-        refresh_interval: Optional[float] = None,
-        vi_mode: Optional[bool] = None,
-        lexer: Optional[Lexer] = None,
-        completer: Optional[Completer] = None,
-        complete_in_thread: Optional[bool] = None,
-        is_password: Optional[bool] = None,
-        key_bindings: Optional[KeyBindingsBase] = None,
-        bottom_toolbar: Optional[AnyFormattedText] = None,
-        style: Optional[BaseStyle] = None,
-        color_depth: Optional[ColorDepth] = None,
-        include_default_pygments_style: Optional[FilterOrBool] = None,
-        style_transformation: Optional[StyleTransformation] = None,
-        swap_light_and_dark_colors: Optional[FilterOrBool] = None,
-        rprompt: Optional[AnyFormattedText] = None,
-        multiline: Optional[FilterOrBool] = None,
-        prompt_continuation: Optional[PromptContinuationText] = None,
-        wrap_lines: Optional[FilterOrBool] = None,
-        enable_history_search: Optional[FilterOrBool] = None,
-        search_ignore_case: Optional[FilterOrBool] = None,
-        complete_while_typing: Optional[FilterOrBool] = None,
-        validate_while_typing: Optional[FilterOrBool] = None,
-        complete_style: Optional[CompleteStyle] = None,
-        auto_suggest: Optional[AutoSuggest] = None,
-        validator: Optional[Validator] = None,
-        clipboard: Optional[Clipboard] = None,
-        mouse_support: Optional[FilterOrBool] = None,
-        input_processors: Optional[List[Processor]] = None,
-        placeholder: Optional[AnyFormattedText] = None,
-        reserve_space_for_menu: Optional[int] = None,
-        enable_system_prompt: Optional[FilterOrBool] = None,
-        enable_suspend: Optional[FilterOrBool] = None,
-        enable_open_in_editor: Optional[FilterOrBool] = None,
-        tempfile_suffix: Optional[Union[str, Callable[[], str]]] = None,
-        tempfile: Optional[Union[str, Callable[[], str]]] = None,
+        editing_mode: EditingMode | None = None,
+        refresh_interval: float | None = None,
+        vi_mode: bool | None = None,
+        lexer: Lexer | None = None,
+        completer: Completer | None = None,
+        complete_in_thread: bool | None = None,
+        is_password: bool | None = None,
+        key_bindings: KeyBindingsBase | None = None,
+        bottom_toolbar: AnyFormattedText | None = None,
+        style: BaseStyle | None = None,
+        color_depth: ColorDepth | None = None,
+        cursor: AnyCursorShapeConfig | None = None,
+        include_default_pygments_style: FilterOrBool | None = None,
+        style_transformation: StyleTransformation | None = None,
+        swap_light_and_dark_colors: FilterOrBool | None = None,
+        rprompt: AnyFormattedText | None = None,
+        multiline: FilterOrBool | None = None,
+        prompt_continuation: PromptContinuationText | None = None,
+        wrap_lines: FilterOrBool | None = None,
+        enable_history_search: FilterOrBool | None = None,
+        search_ignore_case: FilterOrBool | None = None,
+        complete_while_typing: FilterOrBool | None = None,
+        validate_while_typing: FilterOrBool | None = None,
+        complete_style: CompleteStyle | None = None,
+        auto_suggest: AutoSuggest | None = None,
+        validator: Validator | None = None,
+        clipboard: Clipboard | None = None,
+        mouse_support: FilterOrBool | None = None,
+        input_processors: list[Processor] | None = None,
+        placeholder: AnyFormattedText | None = None,
+        reserve_space_for_menu: int | None = None,
+        enable_system_prompt: FilterOrBool | None = None,
+        enable_suspend: FilterOrBool | None = None,
+        enable_open_in_editor: FilterOrBool | None = None,
+        tempfile_suffix: str | Callable[[], str] | None = None,
+        tempfile: str | Callable[[], str] | None = None,
         # Following arguments are specific to the current `prompt()` call.
-        default: Union[str, Document] = "",
+        default: str | Document = "",
         accept_default: bool = False,
-        pre_run: Optional[Callable[[], None]] = None,
+        pre_run: Callable[[], None] | None = None,
         set_exception_handler: bool = True,
+        handle_sigint: bool = True,
         in_thread: bool = False,
     ) -> _T:
         """
@@ -956,6 +969,8 @@ class PromptSession(Generic[_T]):
             self.style = style
         if color_depth is not None:
             self.color_depth = color_depth
+        if cursor is not None:
+            self.cursor = cursor
         if include_default_pygments_style is not None:
             self.include_default_pygments_style = include_default_pygments_style
         if style_transformation is not None:
@@ -1015,10 +1030,12 @@ class PromptSession(Generic[_T]):
         # dumb prompt.
         if self._output is None and is_dumb_terminal():
             with self._dumb_prompt(self.message) as dump_app:
-                return dump_app.run(in_thread=in_thread)
+                return dump_app.run(in_thread=in_thread, handle_sigint=handle_sigint)
 
         return self.app.run(
-            set_exception_handler=set_exception_handler, in_thread=in_thread
+            set_exception_handler=set_exception_handler,
+            in_thread=in_thread,
+            handle_sigint=handle_sigint,
         )
 
     @contextmanager
@@ -1074,52 +1091,53 @@ class PromptSession(Generic[_T]):
         self,
         # When any of these arguments are passed, this value is overwritten
         # in this PromptSession.
-        message: Optional[AnyFormattedText] = None,
+        message: AnyFormattedText | None = None,
         # `message` should go first, because people call it as
         # positional argument.
         *,
-        editing_mode: Optional[EditingMode] = None,
-        refresh_interval: Optional[float] = None,
-        vi_mode: Optional[bool] = None,
-        lexer: Optional[Lexer] = None,
-        completer: Optional[Completer] = None,
-        complete_in_thread: Optional[bool] = None,
-        is_password: Optional[bool] = None,
-        key_bindings: Optional[KeyBindingsBase] = None,
-        bottom_toolbar: Optional[AnyFormattedText] = None,
-        style: Optional[BaseStyle] = None,
-        color_depth: Optional[ColorDepth] = None,
-        include_default_pygments_style: Optional[FilterOrBool] = None,
-        style_transformation: Optional[StyleTransformation] = None,
-        swap_light_and_dark_colors: Optional[FilterOrBool] = None,
-        rprompt: Optional[AnyFormattedText] = None,
-        multiline: Optional[FilterOrBool] = None,
-        prompt_continuation: Optional[PromptContinuationText] = None,
-        wrap_lines: Optional[FilterOrBool] = None,
-        enable_history_search: Optional[FilterOrBool] = None,
-        search_ignore_case: Optional[FilterOrBool] = None,
-        complete_while_typing: Optional[FilterOrBool] = None,
-        validate_while_typing: Optional[FilterOrBool] = None,
-        complete_style: Optional[CompleteStyle] = None,
-        auto_suggest: Optional[AutoSuggest] = None,
-        validator: Optional[Validator] = None,
-        clipboard: Optional[Clipboard] = None,
-        mouse_support: Optional[FilterOrBool] = None,
-        input_processors: Optional[List[Processor]] = None,
-        placeholder: Optional[AnyFormattedText] = None,
-        reserve_space_for_menu: Optional[int] = None,
-        enable_system_prompt: Optional[FilterOrBool] = None,
-        enable_suspend: Optional[FilterOrBool] = None,
-        enable_open_in_editor: Optional[FilterOrBool] = None,
-        tempfile_suffix: Optional[Union[str, Callable[[], str]]] = None,
-        tempfile: Optional[Union[str, Callable[[], str]]] = None,
+        editing_mode: EditingMode | None = None,
+        refresh_interval: float | None = None,
+        vi_mode: bool | None = None,
+        lexer: Lexer | None = None,
+        completer: Completer | None = None,
+        complete_in_thread: bool | None = None,
+        is_password: bool | None = None,
+        key_bindings: KeyBindingsBase | None = None,
+        bottom_toolbar: AnyFormattedText | None = None,
+        style: BaseStyle | None = None,
+        color_depth: ColorDepth | None = None,
+        cursor: CursorShapeConfig | None = None,
+        include_default_pygments_style: FilterOrBool | None = None,
+        style_transformation: StyleTransformation | None = None,
+        swap_light_and_dark_colors: FilterOrBool | None = None,
+        rprompt: AnyFormattedText | None = None,
+        multiline: FilterOrBool | None = None,
+        prompt_continuation: PromptContinuationText | None = None,
+        wrap_lines: FilterOrBool | None = None,
+        enable_history_search: FilterOrBool | None = None,
+        search_ignore_case: FilterOrBool | None = None,
+        complete_while_typing: FilterOrBool | None = None,
+        validate_while_typing: FilterOrBool | None = None,
+        complete_style: CompleteStyle | None = None,
+        auto_suggest: AutoSuggest | None = None,
+        validator: Validator | None = None,
+        clipboard: Clipboard | None = None,
+        mouse_support: FilterOrBool | None = None,
+        input_processors: list[Processor] | None = None,
+        placeholder: AnyFormattedText | None = None,
+        reserve_space_for_menu: int | None = None,
+        enable_system_prompt: FilterOrBool | None = None,
+        enable_suspend: FilterOrBool | None = None,
+        enable_open_in_editor: FilterOrBool | None = None,
+        tempfile_suffix: str | Callable[[], str] | None = None,
+        tempfile: str | Callable[[], str] | None = None,
         # Following arguments are specific to the current `prompt()` call.
-        default: Union[str, Document] = "",
+        default: str | Document = "",
         accept_default: bool = False,
-        pre_run: Optional[Callable[[], None]] = None,
+        pre_run: Callable[[], None] | None = None,
         set_exception_handler: bool = True,
+        handle_sigint: bool = True,
     ) -> _T:
-
         if message is not None:
             self.message = message
         if editing_mode is not None:
@@ -1144,6 +1162,8 @@ class PromptSession(Generic[_T]):
             self.style = style
         if color_depth is not None:
             self.color_depth = color_depth
+        if cursor is not None:
+            self.cursor = cursor
         if include_default_pygments_style is not None:
             self.include_default_pygments_style = include_default_pygments_style
         if style_transformation is not None:
@@ -1203,12 +1223,14 @@ class PromptSession(Generic[_T]):
         # dumb prompt.
         if self._output is None and is_dumb_terminal():
             with self._dumb_prompt(self.message) as dump_app:
-                return await dump_app.run_async()
+                return await dump_app.run_async(handle_sigint=handle_sigint)
 
-        return await self.app.run_async(set_exception_handler=set_exception_handler)
+        return await self.app.run_async(
+            set_exception_handler=set_exception_handler, handle_sigint=handle_sigint
+        )
 
     def _add_pre_run_callables(
-        self, pre_run: Optional[Callable[[], None]], accept_default: bool
+        self, pre_run: Callable[[], None] | None, accept_default: bool
     ) -> None:
         def pre_run2() -> None:
             if pre_run:
@@ -1219,7 +1241,7 @@ class PromptSession(Generic[_T]):
                 # order to run it "soon" (during the next iteration of the
                 # event loop), instead of right now. Otherwise, it won't
                 # display the default value.
-                get_event_loop().call_soon(self.default_buffer.validate_and_handle)
+                get_running_loop().call_soon(self.default_buffer.validate_and_handle)
 
         self.app.pre_run_callables.append(pre_run2)
 
@@ -1343,48 +1365,49 @@ class PromptSession(Generic[_T]):
 
 
 def prompt(
-    message: Optional[AnyFormattedText] = None,
+    message: AnyFormattedText | None = None,
     *,
-    history: Optional[History] = None,
-    editing_mode: Optional[EditingMode] = None,
-    refresh_interval: Optional[float] = None,
-    vi_mode: Optional[bool] = None,
-    lexer: Optional[Lexer] = None,
-    completer: Optional[Completer] = None,
-    complete_in_thread: Optional[bool] = None,
-    is_password: Optional[bool] = None,
-    key_bindings: Optional[KeyBindingsBase] = None,
-    bottom_toolbar: Optional[AnyFormattedText] = None,
-    style: Optional[BaseStyle] = None,
-    color_depth: Optional[ColorDepth] = None,
-    include_default_pygments_style: Optional[FilterOrBool] = None,
-    style_transformation: Optional[StyleTransformation] = None,
-    swap_light_and_dark_colors: Optional[FilterOrBool] = None,
-    rprompt: Optional[AnyFormattedText] = None,
-    multiline: Optional[FilterOrBool] = None,
-    prompt_continuation: Optional[PromptContinuationText] = None,
-    wrap_lines: Optional[FilterOrBool] = None,
-    enable_history_search: Optional[FilterOrBool] = None,
-    search_ignore_case: Optional[FilterOrBool] = None,
-    complete_while_typing: Optional[FilterOrBool] = None,
-    validate_while_typing: Optional[FilterOrBool] = None,
-    complete_style: Optional[CompleteStyle] = None,
-    auto_suggest: Optional[AutoSuggest] = None,
-    validator: Optional[Validator] = None,
-    clipboard: Optional[Clipboard] = None,
-    mouse_support: Optional[FilterOrBool] = None,
-    input_processors: Optional[List[Processor]] = None,
-    placeholder: Optional[AnyFormattedText] = None,
-    reserve_space_for_menu: Optional[int] = None,
-    enable_system_prompt: Optional[FilterOrBool] = None,
-    enable_suspend: Optional[FilterOrBool] = None,
-    enable_open_in_editor: Optional[FilterOrBool] = None,
-    tempfile_suffix: Optional[Union[str, Callable[[], str]]] = None,
-    tempfile: Optional[Union[str, Callable[[], str]]] = None,
+    history: History | None = None,
+    editing_mode: EditingMode | None = None,
+    refresh_interval: float | None = None,
+    vi_mode: bool | None = None,
+    lexer: Lexer | None = None,
+    completer: Completer | None = None,
+    complete_in_thread: bool | None = None,
+    is_password: bool | None = None,
+    key_bindings: KeyBindingsBase | None = None,
+    bottom_toolbar: AnyFormattedText | None = None,
+    style: BaseStyle | None = None,
+    color_depth: ColorDepth | None = None,
+    cursor: AnyCursorShapeConfig = None,
+    include_default_pygments_style: FilterOrBool | None = None,
+    style_transformation: StyleTransformation | None = None,
+    swap_light_and_dark_colors: FilterOrBool | None = None,
+    rprompt: AnyFormattedText | None = None,
+    multiline: FilterOrBool | None = None,
+    prompt_continuation: PromptContinuationText | None = None,
+    wrap_lines: FilterOrBool | None = None,
+    enable_history_search: FilterOrBool | None = None,
+    search_ignore_case: FilterOrBool | None = None,
+    complete_while_typing: FilterOrBool | None = None,
+    validate_while_typing: FilterOrBool | None = None,
+    complete_style: CompleteStyle | None = None,
+    auto_suggest: AutoSuggest | None = None,
+    validator: Validator | None = None,
+    clipboard: Clipboard | None = None,
+    mouse_support: FilterOrBool | None = None,
+    input_processors: list[Processor] | None = None,
+    placeholder: AnyFormattedText | None = None,
+    reserve_space_for_menu: int | None = None,
+    enable_system_prompt: FilterOrBool | None = None,
+    enable_suspend: FilterOrBool | None = None,
+    enable_open_in_editor: FilterOrBool | None = None,
+    tempfile_suffix: str | Callable[[], str] | None = None,
+    tempfile: str | Callable[[], str] | None = None,
     # Following arguments are specific to the current `prompt()` call.
     default: str = "",
     accept_default: bool = False,
-    pre_run: Optional[Callable[[], None]] = None,
+    pre_run: Callable[[], None] | None = None,
 ) -> str:
     """
     The global `prompt` function. This will create a new `PromptSession`
@@ -1407,6 +1430,7 @@ def prompt(
         bottom_toolbar=bottom_toolbar,
         style=style,
         color_depth=color_depth,
+        cursor=cursor,
         include_default_pygments_style=include_default_pygments_style,
         style_transformation=style_transformation,
         swap_light_and_dark_colors=swap_light_and_dark_colors,
